--[[
Name: PratPlugin-1.0
Revision: $Rev: 33633 $
Author(s): Sylvanaar
Website: http://www.wowace.com/files/index.php?path=Prat/
Documentation: http://www.wowace.com/wiki/Prat/
SVN: http://svn.wowace.com/wowace/trunk/Prat/
Description: Library mixin for addons to support and use Prat
Dependencies: Prat, (implies AceLibrary, AceOO-2.0, AceEvent-2.0)
]]

local MAJOR_VERSION = "PratPlugin-1.0"
local MINOR_VERSION = "$Revision: 33633 $"

if not AceLibrary then error(MAJOR_VERSION .. " requires AceLibrary.") end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end
if not AceLibrary:HasInstance("AceOO-2.0") then error(MAJOR_VERSION .. " requires AceOO-2.0.") end

local AceEvent = AceLibrary:HasInstance("AceEvent-2.0") and AceLibrary("AceEvent-2.0")
local AceOO = AceLibrary("AceOO-2.0")

local PratPlugin = AceOO.Mixin {
    "RegisterModule",
    "GetModuleName",
    "RegisterFrame",
    "GetExternalMessageTable",
    "FormatTextForFrame",
    "RegisterMessageItem",   
    "UnregisterMessageItem",
    "BuildExternalMessage",   
}
 
 
-- PratPlugin.moduleName
-- PratPlugin.moduleDesc
-- PratPlugin.consoleName
-- PratPlugin.guiName
-- PratPlugin.Categories
-- PratPlugin.moduleOptions
-- PratPlugin.defaultDB
-- PratPlugin.toggleOptions
-- PratPlugin.revision = tonumber(string.sub("$Revision: 33633 $", 12, -3))
    
    
local PRAT_MODULE = nil
local module = nil
local DEFAULT_DB = {
	on	= false, 
}

local EXTERNAL_MESSAGE = {}
local EXTERNAL_INFO = {r = 1.0, g = 1.0, b = 1.0, id = 0 }

function PratPlugin:RegisterModule(name, desc, console, gui, categories, 
                                    options, templates, defaults, revision) 
    -- create a module instance
    PRAT_MODULE = Prat:RequestModuleName(name)        
    module = Prat:NewModule(PRAT_MODULE)
    
    -- set up the instance data
    module.moduleName = name
    module.moduleDesc = desc
    module.consoleName = console or string.lower(name)
    module.guiName = gui or name
    module.Categories = categories
    module.moduleOptions = options or {}
    module.toggleOptions = templates or {}
    module.defaultDB = defaults or DEFAULT_DB
    module.revision = revision or 0
end
    
function PratPlugin:GetModuleName()
    return PRAT_MODULE
end

function PratPlugin:RegisterFrame(frame)
    -- TODO: Non-Blizz Chat Frame Registration
    -- this value should be used to call FormatTextForFrame
    
    return ChatFrame7
end


-- pass the frame so the correct options are used (ignored currently)
-- the message table should be in prats internal format
function PratPlugin:FormatTextForFrame(frame, message)
    return Prat:PluginRenderAsChatString(frame, message)
end

-- get a message object for use when calling FormatTextForFrame
function PratPlugin:GetExternalMessageTable()
    Prat:ClearChatSections(EXTERNAL_MESSAGE)
    
    EXTERNAL_MESSAGE.LANGUAGE_NOSHOW = "Common"
    EXTERNAL_MESSAGE.INFO = EXTERNAL_INFO

    EXTERNAL_MESSAGE.TYPEPREFIX = nil
    EXTERNAL_MESSAGE.TYPEPOSTFIX = nil
    
    return EXTERNAL_MESSAGE
end

local function safestr(s) return s or "" end

-- message is a table (from GetExternalMessageTable)
-- type is CHAT_MSG_(type), eg GUILD
function PratPlugin:BuildExternalMessage(message, msgtype, channelnum, channelname, player, server, text, r, g, b)
    message.INFO.r, message.INFO.g, message.INFO.b, message.INFO.id = r, g, b, msgtype
    
    if msgtype == "CHANNEL" then 
        message.cC = "["
        message.Cc = "] "
        if type(channelnum) == "number" then 
            message.CHANNELNUM = tostring(channelnum)
            message.CC = ". "
        end
        message.CHANNEL = channelname
    end  

    -- set the player
    if type(player) == "string" and strlen(player)>0 then
        message.pP = "["
        message.lL = "|Hplayer:"
        message.PLAYERLINK = player
        message.LL = "|h"
        message.PLAYER = player
        message.Ll = "|h"
        message.Pp = "]"
    end
    
    if type(server) == "string" then
        s.sS = "-"
        s.SERVER = server        
    end
            
    -- set the message text
    if type(text) == "string" then
        message.MESSAGE = text
    end

    if (message.TYPEPREFIX == nil or message.TYPEPOSTFIX == nil) and type(msgtype) == "string" then
       local chatget = getglobal("CHAT_"..msgtype.."_GET")
 
        if chatget then
            message.TYPEPREFIX, message.TYPEPOSTFIX = string.match(TEXT(chatget), "(.*)%%s(.*)")
            message.TYPEPOSTFIX = safestr(message.TYPEPOSTFIX)
            message.TYPEPREFIX = safestr(message.TYPEPREFIX)            
        end
    end
    
    return message
end


function PratPlugin:RegisterMessageItem(itemname, aftervar)
    Prat:RegisterMessageItem(itemname, aftervar)
end

function PratPlugin:UnregisterMessageItem(itemname)
    Prat:UnregisterMessageItem(itemname)
end
function PratPlugin:AddHandlers()
    function module:GetModuleOptions()
        if type(PratPlugin.OnPratGetModuleOptions) == "function" then
            return PratPlugin:OnPratGetModuleOptions()
        end
    
    	self.moduleOptions = {
    		name	= self.moduleName,
    		desc	= self.moduleDesc,
    		type	= "group",
    		args = {
    	    }
    	}
    	
    	return self.moduleOptions
    end
    
    function module:OnModuleEnable()
        if type(PratPlugin.OnPratModuleEnable) == "function" then
           PratPlugin:OnPratModuleEnable()
        end
    end
    
    -- things to do when the module is disabled
    function module:OnModuleDisable()
        if type(PratPlugin.OnPratModuleDisable) == "function" then
           PratPlugin:OnPratModuleEnable()
        end
    end
end

local function activate(self, oldLib, oldDeactivate)
	PratPlugin = self

	PratPlugin.activate(self, oldLib, oldDeactivate)

	if oldDeactivate then
		oldDeactivate(oldLib)
	end
end

local function external(self, major, instance)
	if major == "AceEvent-2.0" then
		AceEvent = instance
		AceEvent:embed(self)
	end
end

AceLibrary:Register(PratPlugin, MAJOR_VERSION, MINOR_VERSION, activate, nil, external)
PratPlugin = nil